/**************************************************************************
  Module:  Attr.h
  Purpose: An attribute is an Item whose Value is another item.
           Attributes allow the user to have standardized names
           that can be used to access other items.  For instance,
           an "init" attribute could point to one of a number of
           Param Items which could serve as  initializer values.
           Each of the underlying Params may have a different name, 
           but having an "init" attribute  makes that a non-issue.
 ***************************************************************************/

#ifndef __ATTR_H
#define __ATTR_H

#include "KeyBase.h"

///
/// Data private to an Attr.
/// The descendent of ValueBase providing data encapsulation specific to class
/// Attr which is a descendent of the parallel KeyBase hierarchy.
///
class AttrPrivate : public ValueBase {
public:
             AttrPrivate ( KeyBase *aItem, KeyBase *creatorItem );
            ~AttrPrivate ( );
public:
   KeyBase  *itemPtr ;
} ;

/* Need an iterator for this class since it will often be scanned */
/* in its entirety */

///
/// Vista Items whose value is another Item.
/// Attributes allow the user to have standardized names that can be used to
/// access other items.  For instance, an "init" attribute could point to one
/// of a number of Param Items which could serve as initializer values.  Each
/// of the underlying Params may have a different name, but having an "init"
/// attribute makes that a non-issue.
///
class Attr : public KeyBase {
public:
                Attr ( const char *name, KeyBase *aItem );
   virtual     ~Attr( );

   KeyBase     *get ( );
   AttrPrivate& value ( );

private:
  // The default constructor and copy operator should not be called.
                Attr();
   Attr        &operator=(const Attr &) ;
} ;

inline AttrPrivate::AttrPrivate ( KeyBase *aItem, KeyBase *creatorItem ) :
                    ValueBase(creatorItem)
{
   itemPtr   = aItem ;
   m_valueType = AttrItem ;
}

inline AttrPrivate::~AttrPrivate ( )
{
   if (!itemPtr->isNil()) {
      if(--(itemPtr->m_keyCount) == 0)
         delete itemPtr ;
   }
}

inline Attr::Attr ( const char *name, KeyBase *aItem ) :  KeyBase(name)
{  m_valuePtr = (ValueBase *) new AttrPrivate(aItem, this) ;
} 

inline Attr::~Attr()
{ /* do nothing */ }

inline AttrPrivate& Attr::value ( )
{  return *((AttrPrivate *)m_valuePtr);
}

inline KeyBase *Attr::get ( )
{  return value().itemPtr;
}

#endif
