#ifndef __CP_H
#define __CP_H

#include "MemManage.h"

///
/// A counted pointer class.
/// Implements reference counting. Used by IndexSet class only.
///
class CP_t
{
public:
   CP_t ( void *data_, bool priv_ = false )  ;
   ~CP_t ( ) ;
   bool      unlink ( );
public:
   void*        data ;  /* pointer to some data */
   int          count ;   /* number of people that share the pointer */
   bool      priv ;   /* is the data private or shared? */
} ;

inline CP_t::CP_t ( void *data_, bool priv_ )
{
   data  = data_ ;
   count = 1 ;
   priv  = priv_ ;
}

inline CP_t::~CP_t ( )
{
   if (priv && data != 0)
      memFree(&data, 0) ;
}

inline bool CP_t::unlink ( )
{
   bool isUnlinked = false ;

   if (--count == 0) {
      delete this ;
      isUnlinked = true ;
   }

   return isUnlinked ;
}

#endif
