
#include "FieldTable.h"
#include "View.h"
#include "Field.h"
#include "VErr.h"

static void *nothing = (void *) NIL ;
static Field empty("Nil", 0, OTC_UNKNOWN, 1, &nothing) ;

Field *FieldTable::field(const char *name)
{
   Field *fe = (Field *) get(name) ;
   if (fe->isNil()) {
#if DEBUG
      if (VErrGetMask() & VERR_NOTFOUND) {
          char msg[80] ;
          sprintf(msg, "Field %s not found", name) ;
          VErrWarn(msg, VERR_NOTFOUND) ;
      }
#endif
      return &empty ;
   }
   else
      return fe ;
}

Field *FieldTable::createInt(const char *name, int depth, int **data)
{
   Field *fe = (Field *) get(name) ;

   if (fe->isNil()) {
      View *nView ;
      const char *nName ;
      VisPath(m_enclosingView, name, &nView, &nName) ;
      if (!nView->isNil() && nName[0] != '\0') {
         View *isView = nView ;
         while (isView->value().passthrough && isView->m_enclosingTable != NIL)
            isView = isView->m_enclosingTable->m_enclosingView ;

         int mlen = isView->indexSet()->maxLength() ;
         Field *entry = new Field(nName, mlen, OTC_INT, depth,
                                 (void **)data,
                                 (TableBase *)nView->fields()) ;
         nView->fields()->add(entry) ;
         return entry ;
      }
      else {
#if DEBUG
         if (VErrGetMask() & VERR_NOTCREATED) {
             char msg[80] ;
             sprintf(msg, "Could not create field %s", name) ;
             VErrWarn(msg, VERR_NOTCREATED) ;
         }
#endif
         return &empty ;
      }
   } 
   else {
      if (VErrGetMask() & VERR_CONVERSION) {
         if (fe->type() != OTC_INT) {
            char msg[80] ;
            sprintf(msg, "Non integer Field %s not recreated as int", name) ;
            VErrFatal(msg, VERR_CONVERSION) ;
            return fe ;
         }
      }
#if DEBUG
      if (VErrGetMask() & VERR_RECREATED) {
          char msg[80] ;
          sprintf(msg, "Attempting to recreate Field %s...", name) ;
          VErrWarn(msg, VERR_RECREATED) ;
      }
#endif
      return fe->makeSpace() ;
   }
}

Field *FieldTable::createReal(const char *name, int depth, double **data)
{
   Field *fe = (Field *) get(name) ;

   if (fe->isNil()) {
      View *nView ;
      const char *nName ;
      VisPath(m_enclosingView, name, &nView, &nName) ;
      if (!nView->isNil() && nName[0] != '\0') {
         View *isView = nView ;
         while (isView->value().passthrough && isView->m_enclosingTable != NIL)
            isView = isView->m_enclosingTable->m_enclosingView ;

         int mlen = isView->indexSet()->maxLength() ;
         Field *entry = new Field(nName, mlen, OTC_REAL, depth,
                                 (void **) data,
                                 (TableBase *)nView->fields()) ;
         nView->fields()->add(entry) ;
         return entry ;
      }
      else {
#if DEBUG
         if (VErrGetMask() & VERR_NOTCREATED) {
             char msg[80] ;
             sprintf(msg, "Could not create field %s", name) ;
             VErrWarn(msg, VERR_NOTCREATED) ;
         }
#endif
         return &empty ;
      }
   }
   else {
      if (VErrGetMask() & VERR_CONVERSION) {
         if (fe->type() != OTC_REAL) {
            char msg[80] ;
            sprintf(msg, "Non real Field %s not recreated as type real", name) ;
            VErrFatal(msg, VERR_CONVERSION) ;
            return fe ;
         }
      }
#if DEBUG
      if (VErrGetMask() & VERR_RECREATED) {
          char msg[80] ;
          sprintf(msg, "Attempting to recreate Field %s...", name) ;
          VErrWarn(msg, VERR_RECREATED) ;
      }
#endif
      return fe->makeSpace() ;
   }
}

Field *FieldTable::createDynamic(const char *name, void (*f)(KeyBase *item),
                                  int depth)
{
   Field *fe = (Field *) get(name) ;

   if (fe->isNil()) {
      View *nView ;
      const char *nName ;
      VisPath(m_enclosingView, name, &nView, &nName) ;
      if (!nView->isNil() && nName[0] != '\0') {
         Field *entry = new Field(name, f, depth, (TableBase *) this) ;
         nView->fields()->add(entry) ;
         return entry ;
      }
      else {
#if DEBUG
         if (VErrGetMask() & VERR_NOTCREATED) {
             char msg[80] ;
             sprintf(msg, "Could not create field %s", name) ;
             VErrWarn(msg, VERR_NOTCREATED) ;
         }
#endif
         return &empty ;
      }
   }
   else
      return fe ;  /* substitute new function pointer? */
}

