
#ifndef __FIELDTABLE_H
#define __FIELDTABLE_H

#include "Vista.h"
#include "TableBase.h"
#include "MemManage.h"
#include "MemBlock.h"

///
/// The class which provides hashtable management for Fields.
/// FieldTable contains two hash tables, itemTable and attrTable.  The
/// itemTable returns Fields given the name of the field as a key.  The
/// attrTable returns a Group given the name of an attribute.  This Group
/// contains a GroupPrivate which is an array of pointers which  points to the
/// Fields in itemTable having the attribute associated with the group.  In
/// this way, FieldTable provides a means to quickly access fields either by
/// name or by attribute from the same class.
///
class FieldTable : public TableBase {
public:
            FieldTable ( View *v, TableBase *parent );
           ~FieldTable ( );
   Field*   field ( const char *name );
   Field*   createInt ( const char *name, int depth = 1, int **data = NIL );
   Field*   createReal ( const char *name, int depth = 1, double **data = NIL );
   Field*   createDynamic ( const char *name, void (*f)(KeyBase *item)
                          , int depth = 1
                          );
} ;

inline FieldTable::FieldTable ( View *v, TableBase *parent )
                              : TableBase(v, parent, FieldItem)
{
}

inline FieldTable::~FieldTable ( )
{
}

#endif /* __FIELDTABLE_H */

