#ifndef __MEMBLOCK_H
#define __MEMBLOCK_H

/* Note that the MemBlock as currenly implemented does not make */
/* alot of sense.  In reality, the CP (Counted Pointer) data type */
/* does much of what the MemBlock is meant to do.  We could replace */
/* the 'count', 'priv', and 'dataPriv' members below by a pointer */
/* to a CP type, but this would just mean one more level of indirection */
/* when trying to get from the Item level down to the actual data */
/* stored in the Item. */

///
/// A class designed to hold a reference counted block of bulk field data.
/// This class is only used within FieldPrivate at the moment and is quite
/// similar to class CP.  What this class will eventually become is a
/// place to handle arena allocation for multiple fields.  These fields
/// should work in an interleaved or flat configuration, i.e. xyzxyzxyz...
/// or xx...xyy...yzz...z
///
class MemBlock {
public:
           MemBlock ( void* buf, int n, int span, bool isPrivate = true );
          ~MemBlock ( );
   bool unlink( ) ;

public:
   void*   dataPriv; /* block of data */
   int     len;      /* duplicated from view */
   int     maxlen;
   int     count;    /* number of ValueFields sharing this block */
   int     unitSize; /* entity size (bytes) for expansion/contraction */
   bool priv;     /* does the data need free() when count == 0? */
   /* Group*  owners;   Items that share the block */
} ;

inline MemBlock::MemBlock ( void* buf, int n, int span, bool isPrivate
                          ) : dataPriv(buf)
                            , len(n)
                            , maxlen(len)
                            , count(1)
                            , unitSize(span)
                            , priv(isPrivate)
                            // later...
                            // , owners(NIL)
{
}

inline MemBlock::~MemBlock ( )
{  if (priv && dataPriv != 0)
      memFree(&dataPriv, 0);
}

inline bool MemBlock::unlink( )
{
   bool isUnlinked = false ;
   if (--count == 0) {
      delete this ;
      isUnlinked = true ;
   }
   return isUnlinked ;
}

#endif /* __MEMBLOCK_H */
