/**************************************************************************
  Module:  MemManage
  Purpose: This module basically contains a bunch of wrappers for the
  *        malloc() statement.
  ***************************************************************************/

#ifndef __MemManage_h
#define __MemManage_h

#ifdef __cplusplus
extern "C" {
#endif

/* Make this #define LOG_MALLOC if you want to use the malloc logger
 * package (or add it to CFLAGS in the Makefile) */
/* #undef  LOG_MALLOC */
/* #ifndef LOG_MALLOC */
/* #define LOG_MALLOC */
/* #endif */

/* For instructions on how to use the LOG_MALLOC package for detecting
 * memory leaks and the high water mark in your code, see the file
 * MemManageLog.h */ 


/* Leave this include here - it will get the #define for LOG_MALLOC
 * from MemManageLog.h and redefine functions below as needed */



/* If 1, then calls that use memAllocBlock[Doubles|Ints] will do a
 * single allocate for all the variables in the list.
 * NOTE: Make sure that you match the argument lists in the
 * memAllocBlock* and memBlockFree() calls */
#define BLOCK_MALLOCS 1   


/* If 1, makes sure that all mallocs are done about equal size, thus
 * decreasing the risk of fragmentation.
 * Allowable block sizes (in bytes) are:
 * 32, 128, 256, 512, 1024
 * and then every 1024 boundary beyond that. */
#define ALIGN_MALLOCS 0


/* If 1, outputs a message when a realloc moves a pointer to a new
 * location.  */
#define NOISY_REALLOC 0


/* If we're not blocking together malloc requests, then just
 * send the block requests straight to the regular individual malloc
 * allocator */
#if !(BLOCK_MALLOCS)
# define memAllocBlockDoubles memAllocDoubles
# define memAllocBlockInts    memAllocInts
# define memBlockFree         memFree
#endif


#define INITIAL_BUCKET_SIZE 50
#define GROW_BUCKET_SIZE 20


#define mallocInt(x) (int*)malloc((x)*sizeof(int))
#define mallocDouble(x) (double*)malloc((x)*sizeof(double))


void MemManageInit(void) ;
void memAllocDoubles(int size, ...) ;
void memAllocInts(int size, ...) ;
void memReallocInts(int newsize, ...) ;
void memReallocDoubles(int newsize, ...) ;
void memAllocIntPointers(int size, ...) ;
void memFree(void*, ...) ;

void  memSetAllocGrowthPct(double growPct) ;
void  memSetShrinkFreq(int cycs) ;
void  memSetShrinkMult(double shrinkMult) ;
double memGetAllocGrowthPct(void) ;
int   memGetShrinkFreq(void) ;
double memGetShrinkMult(void) ;
int   memCalcAllocGrowth(int size) ;
int   memCalcAllocShrink(int currCycle, int actual, int actualLast, int alloc) ;

#if BLOCK_MALLOCS
void memAllocBlockInts(int size, ...) ;
void memAllocBlockDoubles(int size, ...) ;
void memBlockFree(void*, ...) ;
#endif

#ifdef __cplusplus
}
#endif

#endif
