
#include "silo.h"
#include "ViewIO.h"

/***********************************************************************
 * Class  : SILOReader
 * Author(s) : Keasler
 * Purpose   : Set up a data transfer class for Vista I/O
 **********************************************************************/

///
/// A class to implement reading from the Silo external persistent format into
/// Vista.
/// This class implements all the pure virtual functions of class VistaTransfer.
///
class SILOReader : public VistaTransfer {
   DBfile *file ;

public:
   SILOReader(const char *fileName, int errLevel = 0)
   {
      if ((file = DBOpen( fileName, DB_PDB, DB_READ)) == NULL)
         setError(VIO_ERR_FOPEN) ;
      else 
         DBShowErrors(((errLevel == 0) ? DB_NONE : DB_ALL), NULL) ;
   }

   ~SILOReader() { DBClose(file) ; }

   void EnterScope(const char *name)
   {
      if (DBSetDir(file, const_cast<char *> (name)) == -1)
         setError(VIO_ERR_SETSCOPE) ;
   }

   void ArrayReal(const char *name,
                            double *data, int length, int stride)
   {
      if (stride == 0 || stride == sizeof(double)) {
         if (DBReadVar(file, const_cast<char *> (name), data) == -1)
            setError(VIO_ERR_READ) ;
      }
      else /* strided */ {
         double *buf = new double[length] ;
         if (DBReadVar(file, const_cast<char *> (name), buf) == -1)
            setError(VIO_ERR_READ) ;
         else {
            int i ;
            for (i=0; i<length; ++i) {
               *data = buf[i] ;
               data = ((double *)((char *) data + stride)) ;
            }
         }
         delete [] buf ;
      }
   }

   void ArrayInt(const char *name, int *data, int length, int stride)
   {
      if (stride == 0 || stride == sizeof(int)) {
         if (DBReadVar(file, const_cast<char *> (name), data) == -1)
            setError(VIO_ERR_READ) ;
      }
      else /* strided */ {
         int *buf = new int[length] ;
         if (DBReadVar(file, const_cast<char *> (name), buf) == -1)
            setError(VIO_ERR_READ) ;
         else {
            int i ;
            for (i=0; i<length; ++i) {
               *data = buf[i] ;
               data = ((int *)((char *) data + stride)) ;
            }
         }
         delete [] buf ;
      }
   }

   void String(const char *name, char *data, int length)
   {
      if (DBReadVar(file, const_cast<char *> (name), data) == -1)
         setError(VIO_ERR_READ) ;
   }

   int Length(const char *name)
   {
      int varlen = DBGetVarLength(file, const_cast<char *> (name)) ;
      if (varlen == -1)
         setError(VIO_ERR_READ) ;
      return varlen ;
   }

   void ExitScope(void)
   {
      if (DBSetDir(file, const_cast<char *> ("..")) == -1)
         setError(VIO_ERR_SETSCOPE) ;
   }
} ;

