/************************************************************************
 * File    : VErr.h
 * Purpose : Allow user to register an error handler
 ************************************************************************/

#ifndef __VERR_H
#define __VERR_H

/* Vista error conditions */
#define VERR_NOTFOUND          0x0001
#define VERR_NOTCREATED        0x0002
#define VERR_RECREATED         0x0004
#define VERR_CONVERSION        0x0008
#define VERR_TRUNCATION        0x0010
#define VERR_AMBIGUOUS_CREATE  0x0020
#define VERR_INTERNAL          0x0040

/* Vista error levels */
#define VERR_WARN   1
#define VERR_FATAL  2

#define VErrEnable(mask) VErrSetMask(VErrGetMask() | (mask)) 
#define VErrDisable(mask) VErrSetMask(VErrGetMask() & ~(mask))

/*
 The user can register an error handler function that will be called
 when a Vista error occurs.  Error handler arguments are:
   A char[] error message produced by Vista
   integer error level (see above)
   integer error condition (some union subset of the constants above)
*/
void VErrHandler(void (*f)(const char *, int, int)) ;

void VErrWarn(const char *msg, int error) ;
void VErrFatal(const char *msg, int error) ;

/* vistaErrMask used to be static to VErr.c and the macros were funcs */
extern int vistaErrMask ;  

/* Use VErrSetMask to tell Vista which errors you want to catch */

#define VErrSetMask(val) (vistaErrMask = (val))
#define VErrGetMask()   vistaErrMask

#endif  /* __VERR_H */
