/**************************************************************************
  Module:  HashADT
  Purpose: Hash table implementation for Zonevar variables with some of
           the code also being used for the Scalars_t hash table.
 ***************************************************************************/



#ifndef __VHASHTABLE_H
#define __VHASHTABLE_H

int  VHashTableNumEntries(VHashTable *hashTable) ;

#include "KeyBase.h"

/* HashTable_t */
///
/// A hashtable mapping names to Items.
/// This class allows multiple keys to refer to the same value.
///
class VHashTable {
public:
   KeyBase **m_hashHead ;
   int       m_tableSize ;
   int       m_totalHashed ;
} ;

/* HashTraverse_t */
///
/// A traverser over the entries in a VHashTable.
///
class VHashTraverse {
public:
   int ndx ;
   KeyBase *currEntry ;
} ;

/* These values are chosen because (as prime numbers)
 * they should reduce hash collisions */
typedef enum _vhashTableSizes {
   VTINY_HASH_SIZE  = 11,
   VSMALL_HASH_SIZE = 47,
   VBIG_HASH_SIZE   = 103,
   VHUGE_HASH_SIZE  = 317
} VHashTableSizes_t ;



VHashTable *VHashTableNew(VHashTableSizes_t tableSize);
void VHashTableInsert(VHashTable *hashTable, KeyBase *item) ;
KeyBase *VHashTableRemoveEntry(VHashTable *table, const char *str,
                               bool kill = false) ;
KeyBase *VHashTableFind(VHashTable  *hashTable, const char *str) ;
void VHashTableFree(VHashTable  **table) ;
void VHashTableFreeBase(VHashTable  **table) ;
void VHashTableExamine(VHashTable  *hashTable) ;

/*
  These functions below should be used for traversing through every 
  entry in the hash table.  The reason for the HashTraverse_t object
  is thread safety.  It is an opaque handle which keeps track of
  where you currently are in the table, as well as a reference to
  the current table item, which should be extracted with
  HashTraverseEntry()

  The basic way of traversing a table is:

  HashTraverse_t hash = HashTraverseInit() ;
  while (HashTraverseGetNext(hashTable, hash) == true) {
     char *whatever = (char*)HashTraverseEntry(hash) ;
     ...
  }
  HashTraverseFree(hash) ;


  Make sure that you reinitialize the HashTraverse_t object before
  starting the traversal of another hash table.  Also remember to
  free the HashTraverse_t object after the traversal loop
*/

VHashTraverse *VHashTraverseInit(void) ;
bool VHashTraverseGetNext(VHashTable  *hashTable, VHashTraverse *axs) ;
KeyBase *VHashTraverseEntry(VHashTraverse *axs) ;
void VHashTraverseFree(VHashTraverse *axs) ;

#endif  /* __VHASHTABLE_H */
