
#include "View.h"
#include <string.h>

int VStrlen(const char *name)
{
   return (name == NIL) ? 0 : strlen(name) ;
}

char *VStrdup(const char *str)
{
   char *s ;
   int len ;

   if (str == NULL)
      return NULL ;
   else {
      len = strlen(str) ;
      s = (char*)malloc(len+1) ;
      strncpy(s, str, len+1) ;
      return s ;
   }
}


/************************************************************************
 * Function  : BinarySearchLess
 * Author(s) : Jeff Keasler
 * Purpose   : Same as BinarySearch, but if `num' is not in `map' then
 *             the index of the next smallest item to `num' is returned.
 ************************************************************************/
int VBinarySearchLess(int *map, int start, int mapSize, int num)
{
   int low, mid, high ;
   int oldMid ;

   mid = low = start ;
   oldMid = high = start+mapSize ;

   while (mid != oldMid)
   {
      oldMid = mid ;
      mid = (low+high)/2 ;

      if (map[mid] < num)
         low = mid ;
      else if (map[mid] > num)
         high = mid ;
      else
         return mid ;
   }

   return mid ;
}

void VisPath(View *inView, const char *inPath,
             View **outView, const char **outPath)
{
   char tmpPath[256] ;
   char *tmpPtr = tmpPath ;
   const char *scan = inPath ;

   /* use a ':' at beginning if path based? */

   /* if there is a leading '/', go to root */
   if (*scan == '/') {
      inView = inView->value().rootView ;
      char *rootName = inView->name() ;
      inPath = ++scan ;
      while (*rootName != 0)
         if (*scan == *rootName) {
            ++scan ;
            ++rootName ;
         }
      if (*scan == '/')
         ++scan ;
   }
      
   /* Interpret each occurrence of '<name>/' as a view */
   /* that should be moved into.  The remaining suffix */
   /* will be returned by this function */
   while (*scan != '\0') {
      if (*scan != '/')
         *tmpPtr++ = *scan++ ;
      else {
         *tmpPtr = '\0' ;
         if (tmpPath[0] == '.') {

            /* if there is a '../' move up to parent view */
            if (tmpPath[1] == '.') {
               if (tmpPath[2] == '\0') {      /*  move up a view */  ;
                  TableBase *ct = inView->m_enclosingTable ;
                  if (ct != NIL)
                     inView = ct->m_enclosingView ;
               }
               else
                  inView = inView->view(tmpPath) ;
            }
            else /* if there is a './' ignore it */ {
               if (tmpPath[1] == '\0')
                  /* (Warn about doing something wasteful?) */ ;
               else
                  inView = inView->view(tmpPath) ;
            }
         }
         else /* path component is not "./" or "../" */ {
            inView = inView->view(tmpPath) ;
         }
         tmpPtr = tmpPath ;
         inPath = ++scan ;
      } /* end of path component processing */
   } /* end of path processing */

   if (inPath[0] == '.') {

      /* if there is a '../' move up to parent view */
      if (inPath[1] == '.') {
         if (inPath[2] == '\0') {      /*  move up a view */  ;
            TableBase *ct = inView->m_enclosingTable ;
            if (ct != NIL)
               inView = ct->m_enclosingView ;
            inPath += 2 ;
         }
      }
      else /* if there is a './' ignore it */ {
         if (inPath[1] == '\0')
            ++inPath ;
      }
   }

   *outView = inView ;
   *outPath = inPath ;

   return ;
}

/************************************************************************
 * Function  : IntersectNodes2
 * Author(s) : Rob Neely, Tim Pierce
 * Purpose   : Given two arrays of integers in ascending order, this
 *             routine returns the number of matching elements, and
 *             two arrays of indices: the indices in the first array
 *             at which intersection occurs, and the corresponding set
 *             of indices in the second array.
 *             This is a slight modification of IntersectNodes, to return
 *             indices into both input arrays.
 ************************************************************************/

void VIntersectNodes2 (int *arr1, int size1, int *arr2, int size2,
                      int **matches1, int **matches2, int *numMatches)
{
   int smaller ;
   int i, j;


   smaller = (size1 < size2) ? size1 : size2 ;
   *numMatches = 0 ;
   if (smaller == 0) {
      *matches1 = NULL ;
      *matches2 = NULL ;
      return ;
   }
   else
      memAllocInts(smaller, matches1, matches2, NULL) ;

   /* This algorithm assumes that the nodelists are sorted */
   i=j=0 ;

   for (;;) {
      if ((i >= size1) || (j >= size2))
         break ;
      while((i < size1) && (arr1[i] < arr2[j]))
         i++ ;
      if (i >= size1)
         break ;
      while((j < size2) && (arr2[j] < arr1[i]))
         j++ ;
      if (j >= size2)
         break ;
      if (arr1[i] == arr2[j]) {
         (*matches1)[(*numMatches)] = i ;
         (*matches2)[(*numMatches)] = j ;
         (*numMatches)++ ;
         i++ ;
         j++ ;
      }
      else if (arr1[i] < arr2[j])
         i++ ;
      else if (arr2[j] < arr1[i])
         j++ ;
   }
   /* change the size of the array */
   /* (reallocing to a size of zero should be the same as freeing
    * the object, but insight doesn't seem to think so... hence
    * the extra check here with an explicit free */
   if (*numMatches == 0) {
      memFree(matches1, matches2, NULL) ;
   }
   else {
      memReallocInts(*numMatches, matches1, matches2, NULL) ;
   }
}

