#include "Vista.h"
#include "View.h"
#include "VUtil.h"
#include "MemBlock.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/* Forward declaration */

static void calcPolar(KeyBase *component) ;

int main(int argc, char *argv[])
{
    int i ;

    View node("node", new IndexSet(10)) ;

    /* create static x and y fields, and get a pointer to their data */
    double *x = node.fieldCreateReal("x")->Real() ;
    double *y = node.fieldCreateReal("y")->Real() ;

    /* create dynamic r and theta fields */
    node.fieldCreateDynamic("r", calcPolar) ;
    node.fieldCreateDynamic("theta", calcPolar) ;

    /***************   First Data Set   ***********************/

    /* create some random x, y coordinates */
    for (i=0; i<10; ++i)  {
       x[i] = (double) (rand() % 100) ;
       y[i] = (double) (rand() % 100) ;
    }

    /* ask for r, theta coordinates */
    double *r = node.fieldReal("r") ;
    double *theta = node.fieldReal("theta") ;

    printf("\npoint         x            y            r          theta\n") ;
    for (i=0; i<10; ++i)
       printf(" %2d    %12.6f %12.6f %12.6f %12.6f\n",
              i, x[i], y[i], r[i], theta[i]) ;

    /***************  Second Data Set   ***********************/

    /* create some different random x, y coordinates */
    for (i=0; i<10; ++i)  {
       x[i] = (double) (rand() % 100) ;
       y[i] = (double) (rand() % 100) ;
    }

    /* ask for r, theta coordinates */
    r = node.fieldReal("r") ;
    theta = node.fieldReal("theta") ;

    printf("\npoint         x            y            r          theta\n") ;
    for (i=0; i<10; ++i)
       printf(" %2d    %12.6f %12.6f %12.6f %12.6f\n",
              i, x[i], y[i], r[i], theta[i]) ;

    return 0 ;
}

/*****************************************************************
  * calculate polar coordinates from rectilinear coordinates.
  *
  * Note that the interface to this callback function could be made
  * cleaner.
  *
  * The component argument is the object being recomputed.
  * Since every object has a name, we can query the component name
  * and take the appropriate action.
  ****************************************************************/

static void calcPolar(KeyBase *component)
{
    int i ;
    Field *pos = reinterpret_cast<Field *> (component) ;
    FieldPrivate *priv = &pos->value() ;
    double *result ;

    /* figure out which view they are trying to pull data from */

    View *currView = pos->m_enclosingTable->m_enclosingView ;
    int len = currView->length() ;
    if (priv->block[priv->base] == NIL) {
       priv->fieldType = OTC_REAL ;
       priv->len = len ;
       priv->block[priv->base] = new MemBlock(NIL, 0, sizeof(double)) ;
    }
    priv->makeSpace() ;
    result = pos->Real() ;

    /* get the closest x and y coordinates visible to their view */

    real8 *x = currView->fieldReal("x") ;
    real8 *y = currView->fieldReal("y") ;

    if (strcmp(pos->name(), "r") == 0) {

       for (i=0; i<len; ++i)
          result[i] = sqrt(x[i]*x[i] + y[i]*y[i]) ;
    }
    else if (strcmp(pos->name(), "theta") == 0) {
       for (i=0; i<len; ++i) {
          if (y[i] != 0)
             result[i] = atan(y[i]/x[i])*180.0/M_PI ;
          else
             result[i] = 0 ;
       }
    }

    return ;
}
