
#include "View.h"

/***********************************************
 *
 *   solve Laplacian(u) = f by Jacobi method
 *
 ***********************************************/

#define SIDE         9     /* length of side of the solution subgrid */
#define BC_SIDE  (SIDE+2)  /* length of side of the solution + BC grid */

int main(int argc, char *argv[])
{
   /* create a 2D structured 11x11 grid for inital+boundary conditions */
   const int dims = 2 ;
   int extents[dims] = { BC_SIDE, BC_SIDE } ;
   int strides[dims] = {  1, BC_SIDE } ;
   int baseOffset    = 0 ;
   IndexSet *grid = new IndexSet(dims, extents, strides, baseOffset) ;
   View *poisson = new View("Poisson Problem", grid) ;

   poisson->fieldCreateReal("u", 2) ; /* 2 timesteps worth of data */
   poisson->field("u")->fill(0.0) ;   /* u_old */
   poisson->field("u")->advance(1) ;
   poisson->field("u")->fill(0.0) ;   /* u_new */
   poisson->field("u")->advance(1) ;

   /* create a 2D structured 9x9 subgrid where solution is formed */
   extents[0] = extents[1] = SIDE ;
   baseOffset = strides[0] + strides[1] ; /* grid corner in parent indexspace */
   grid = new IndexSet(dims, extents, strides, baseOffset) ;
   View *subgrid = poisson->viewCreate("solution grid", grid) ;

   subgrid->fieldCreateReal("f")->fill(0.0) ;
   subgrid->fieldReal("f")[subgrid->length()/2] = 1.0 ; /* spike at center */
   
   int width = grid->m_extents[0] ;
   int height = grid->m_extents[1] ;

   int width_stride = grid->m_strides[0] ;
   int height_stride = grid->m_strides[1] ;

   real8 dh2 = 1.0 / (double) (SIDE*SIDE) ;
   
   for (int n=1; n<750; ++n) {
      int       f_index = 0 ;   /* subgrid index space */
      real8 *   f       = subgrid->fieldReal("f") ;

      int    u_index  = grid->m_parentCornerOffset ; /* poisson index space */
      real8 *u_old    = poisson->fieldReal("u", n-1) ;
      real8 *u_new    = poisson->fieldReal("u", n) ;

      for (int i=0; i<height; ++i) {
         int tmp = u_index ;
         for (int j=0; j<width; ++j) {
            u_new[u_index] = - 0.25 * ( dh2 * f[f_index++] -
                                      u_old[u_index - width_stride] -
                                      u_old[u_index + width_stride] -
                                      u_old[u_index - height_stride] -
                                      u_old[u_index + height_stride]) ;
            u_index += width_stride ;
         }
         u_index = tmp + height_stride ;
      }
   }

   printf("u(0.5, 0.5) = %.14f\n",
          poisson->fieldReal("u")[poisson->length()/2]) ;
      
   delete poisson ;

   return 0 ;
}
