#ifndef __PARAMTABLE_H
#define __PARAMTABLE_H

#include "Vista.h"
#include "Param.h"
#include "TableBase.h"

///
/// The class which provides hashtable management for Params.
/// ParamTable contains two hash tables, itemTable and attrTable.  The
/// itemTable returns Params given the name of the param as a key.  The
/// attrTable returns a Group given the name of an attribute.  This Group
/// contains a GroupPrivate which is an array of pointers which  points to the
/// Params in itemTable having the attribute associated with the group.  In
/// this way, ParamTable provides a means to quickly access params either by
/// name or by attribute from the same class.
///
class ParamTable : public TableBase {
public :
           ParamTable ( View* v, TableBase* parent );
          ~ParamTable ( );
   Param*  param ( const char *name );
   Param*  setInt ( const char *name, int value );
   Param*  setReal ( const char *name, double value );

   Param*  setString ( const char *name, const char *value,
                       int n = 0, bool copyMe = VISTA_OWNS );

   Param*  setIntArray ( const char *name, int *value,
                         int n, bool copyMe = VISTA_OWNS);

   Param*  setRealArray ( const char *name, double *value,
                          int n, bool copyMe = VISTA_OWNS);

   int&    paramInt(const char *name) ;
   double& paramReal(const char *name) ;
   Param*  paramSetInt(const char *name, int value) ;
   Param*  paramSetReal(const char *name, double value) ;
};

inline ParamTable::ParamTable ( View *v, TableBase *parent )
                              : TableBase(v, parent, ParamItem)
{
}

inline ParamTable::~ParamTable ( )
{
}

inline int& ParamTable::paramInt(const char *name)
{  return param(name)->Int() ;
}

inline double& ParamTable::paramReal(const char *name)
{  return param(name)->Real() ;
}

inline Param *ParamTable::paramSetInt(const char *name, int value)
{  return setInt(name, value) ;
}

inline Param *ParamTable::paramSetReal(const char *name, double value)
{  return setReal(name, value) ;
}

#endif /* __PARAMTABLE_H */
