#include "Group.h"
#include "Vista.h"
#include "View.h"
#include "VHashTable.h"

#include <stdio.h>
#include <string.h>
#include <ctype.h>

/**************************************************************************
 * Subroutine:  DumpUltra
 * Purpose   :  Dump tagged problem data to an ultra file 
 *************************************************************************/

void DumpUltra(View *prob)
{
   int i ;
   FILE *fp ;
   char fname[100] ;
   char *tail ;

   View *elem = prob->view("elem") ;

   strcpy(fname, prob->name()) ;

   /* Skip past the junk */
   for (tail=fname; isalpha(*tail); ++tail) ;

   sprintf(tail, "_%04d\0", prob->paramInt("cycle")) ;

   if ((fp = fopen(fname, "w")) == NULL)
   {
      printf("Could not open file %s. Aborting.\n", fname) ;
      exit (-1) ;
   }

   fprintf(fp, "# Ultra Plot File\n") ;
   fprintf(fp, "# Problem: %s\n", prob->name()) ;

   for (Group::Iter pi(prob->params()); !pi.end(); pi.next())
   {
      Param *currParam = (Param *) pi.item() ;
      switch(currParam->type())
      {
         case OTC_INT :
         {
            fprintf(fp, "# %s = %d\n", currParam->name(), currParam->Int()) ;
         }
         break ;

         case OTC_REAL :
         {
            fprintf(fp, "# %s = %f\n", currParam->name(), currParam->Real()) ;
         }
         break ;

         default:
            /* skip and warn */
            printf("Add another case in DumpUltra Scalar\n") ;
      }
   }

   for (Group::Iter ei(elem->fields()); !ei.end(); ei.next())
   {
      int plotlen = elem->length() ;
      Field *currField = (Field *) ei.item() ;
      switch(currField->type())
      {
         case OTC_INT :
         {
            int *data = currField->Int() ;
            fprintf(fp, "# %s\n", currField->name()) ;
            for (i=0; i<plotlen; ++i)
               fprintf(fp, "%f %f\n", (double) i, (double) data[i]) ;
            fprintf(fp, "\n") ;
         }
         break ;

         case OTC_REAL :
         {
            double *data = currField->Real() ;
            fprintf(fp, "# %s\n", currField->name()) ;
            for (i=0; i<plotlen; ++i)
               fprintf(fp, "%f %f\n", (double) i, data[i]) ;
            fprintf(fp, "\n") ;
         }
         break ;

         default:
            /* skip and warn */
            printf("Add another case in DumpUltra Scalar\n") ;
      }
   }

   fclose(fp) ;

   return ;
}

